@@ -11,7 +11,12 @@ def is_waf(self):
     if check_schema_01(self):
         return True
 
-    if self.matchCookie(r'^TS.+?'):
+    # ASM ≥ 11.4.0 → eight hex digits after “TS” - https://my.f5.com/manage/s/article/K6850
+    if self.matchCookie(r'TS[a-fA-F0-9]{8}=.+'):
+        return True
+    
+    # ASM 10.0.0 – 11.3.0 → six hex digits after “TS” - https://my.f5.com/manage/s/article/K6850
+    if self.matchCookie(r'TS[a-fA-F0-9]{6}=.+'):
         return True
 
     return False
