@@ -4,7 +4,7 @@ Copyright (C) 2024, WAFW00F Developers.
 See the LICENSE file for copying permission.
 '''
 
-NAME = 'SiteGuard (Sakura Inc.)'
+NAME = 'SiteGuard (EG Secure Solutions Inc.)'
 
 
 def is_waf(self):
